
import dopp;

// helper - is lexeme keyword
export bool isKeyword(string lexeme){
  static immutable keywords = ["if", "else", "while", "for", "return"];
  return keywords.canFind(lexeme);
}


// lexer - Tokenizer makes tokens from text
export Token[] tokenize(string source){
  Token[] tokens;
  int i = 0;
  while(i < source.length){
    
    if(source[i].isWhite){ // skip whitespaces
      i++;
    
    }else if(source[i].isAlpha || source[i] == '_'){
      auto start = i;
      while(i < source.length && (source[i].isAlphaNum || source[i] == '_')){
        i++;
      }
      string lexeme = source[start .. i];
      tokens ~= Token(lexeme.isKeyword ? TokenType.Keyword : TokenType.Identifier, lexeme);
    
    }else if(source[i].isDigit){ // number
      auto start = i;
      while(i < source.length && (source[i].isDigit || source[i] == '_')){ // underscore can be inside number like 5_000 etc
        i++;
      }
      if(i < source.length && source[i] == '.'){ // include dot for float
        i++;
        while(i < source.length && source[i].isDigit){
          i++;
        }
        tokens ~= Token(TokenType.Float, source[start .. i]);
      
      }else{
        tokens ~= Token(TokenType.Integer, source[start .. i]);
      }
    
    }else if(source[i] == '"'){ // type string begins
      auto start = i++; // string begin position
      while(i < source.length && source[i] != '"'){ // goto type string end position
        i++;
      }
      if(i < source.length){ // close quotes
        i++;
      }
      tokens ~= Token(TokenType.String, source[start .. i]);
    
    }else{ // common symbols as tokens
      tokens ~= Token(TokenType.Symbol, source[i].to!string);
      i++;
    }
  }
  return tokens;
}

